--[[
**********************************************************************
Golden Ticket - Freedom from the GM Ticket
**********************************************************************
This file is part of GoldenTicket, a World of Warcraft Addon

GoldenTicket is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

GoldenTicket is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with GoldenTicket.  If not, see <http://www.gnu.org/licenses/>.

**********************************************************************
]]

GoldenTicket = LibStub("AceAddon-3.0"):NewAddon("GoldenTicket", "AceEvent-3.0", "AceTimer-3.0", "AceConsole-3.0")
local db 
local mod = GoldenTicket

local L = LibStub("AceLocale-3.0"):GetLocale("GoldenTicket", false)
local ticketIsOpen 
local LDBIcon = LibStub("LibDBIcon-1.0", true)
local LDB = LibStub("LibDataBroker-1.1") -- LDB required since this is solely an LDB provider
local AC = LibStub("AceConfig-3.0")
local ACD = LibStub("AceConfigDialog-3.0")

local ldbLabelText = L["No ticket"]
local tooltipTextNoTicket =
   L["|cffd9f919Left Click|r to open a new ticket."].."\n"..
   L["|cffd9f919Right Click|r to configure Golden Ticket."]
local tooltipTextTicket =
   "%s\n|cffffffff%s|r\n\n"..
   L["|cffd9f919Left Click|r to edit ticket."].."\n"..
   L["|cffd9f919Middle Click|r to abandon your current ticket."].."\n"..
   L["|cffd9f919Right Click|r to configure Golden Ticket."]
   
local function GT_OnClick(frame, button)
   if button == "LeftButton" then
      HelpFrame_ShowFrame("OpenTicket")
   elseif button == "MiddleButton" then
      StaticPopup_Show("HELP_TICKET_ABANDON_CONFIRM");
   else
      mod:ToggleConfigDialog()
   end   
end

function mod:OnInitialize()
   db = GoldenTicketDB or {
      minimapIcon = {}
   }
   GoldenTicketDB = db

   self.ldb =
      LDB:NewDataObject("Golden Ticket", {
			   label = db.hideLabel and nil or L["Golden Ticket"],
			   icon = "Interface\\HelpFrame\\OpenTicketIcon",
			   type = "data source",
			   tooltiptext =  tooltipTextNoTicket,
			   text = (db.hideText or db.hideTextTicket) and nil or ldbLabelText,
			   OnClick = GT_OnClick
			});
   if LDBIcon then
      LDBIcon:Register("Golden Ticket", self.ldb, db.minimapIcon)
   end
   mod:SetupOptions()
end


function mod:UpdateTicketStatus(event, category, ticketDescription, ticketAge, oldestTicketTime, updateTime, assignedToGM, openedByGM)   
   local text, tooltiptext
   if category then
      ticketIsOpen = true
      if TicketStatusFrame:IsShown() then
	 TicketStatusFrame:Hide()
      end
      if assignedToGM == 0 then
	 text = L["Unassigned"]
      elseif assignedToGM == 1 then
	 text = L["Assigned"]
      elseif assignedToGM == 2 then
	 text = L["Escalated"]
      else
	 text = L["Unknown"]
      end
      tooltiptext = tooltipTextTicket:format(TicketStatusTitleText:GetText(),
					     TicketStatusTime:IsShown() and
						TicketStatusTime:GetText() or "")
   else
      ticketIsOpen = false
      tooltiptext = tooltipTextNoTicket
      text = L["No ticket"]
   end
   ldbLabelText = text
   if db.hideText or (db.hideTextTicket and not ticketIsOpen) then
      text = nil
   end
   if text ~= mod.ldb.text then
      mod.ldb.text = text
   end
   if tooltiptext ~= mod.ldb.tooltiptext then
      mod.ldb.tooltiptext = tooltiptext
   end
end

function mod:OnEnable()
   mod:RegisterEvent("UPDATE_TICKET", "UpdateTicketStatus")
end

function mod:OnDisable()
   if ticketIsOpen then 
      TicketStatusFrame:Show()
   end
   mod:UnregisterEvent("UPDATE_TICKET")
end

function mod:ToggleConfigDialog()
   InterfaceOptionsFrame_OpenToCategory(mod.configFrame)
end


-- Configuration

local options = { 
   general = {
      type = "group",
      name = "Golden Ticket",
      order = 1,
      args = {
	 minimapIcon = {
	    type = "toggle",
	    name = L["Enable minimap icon"],
	    desc = L["Show an icon to handle tickets at the minimap"],
	    get = function() return not db.minimapIcon.hide end,
	    set = function(info, value) db.minimapIcon.hide = not value LDBIcon[value and "Show" or "Hide"](LDBIcon, "Golden Ticket") end,
	    disabled = function() return not LDBIcon end,
	 },
	 hideLabel = {
	    type = "toggle",
	    name = L["Hide text label"],
	    desc = L["Hide the text label in LDB display"],
	    get = function() return db.hideLabel end,
	    set = function(info, value) db.hideLabel = value if value then mod.ldb.label = nil else mod.ldb.label = L["Golden Ticket"] end end
	 },
	 hideText = {
	    type = "toggle",
	    name = L["Hide status text"],
	    desc = L["Hide the ticket status text"],
	    get = function() return db.hideText end,
	    set = function(info, value) db.hideText = value if value or (db.hideTextTicket and not ticketIsOpen) then mod.ldb.text = nil else mod.ldb.text = ldbLabelText end end
	 },
	 hideTextIfNoTicket = {
	    type = "toggle",
	    name = L["Hide no ticket text"],
	    desc = L["Hide the ticket status text when no ticket is currently open"],
	    get = function() return db.hideTextTicket end,
	    set = function(info, value) db.hideTextTicket = value if not ticketIsOpen then  if value then mod.ldb.text = nil else mod.ldb.text = ldbLabelText end end end,
	    disabled = function() return db.hideText end,
	 },
      }
   }
}

function mod:SetupOptions()
   AC:RegisterOptionsTable("Golden Ticket", options.general, "goldenticket")
   mod.configFrame = ACD:AddToBlizOptions("Golden Ticket", L["Golden Ticket"])
end
